#########
# 解説1
#########
import ctypes 
import tisgrabber as tis
import time

#tisgrabber_x64.dllをインポートする
ic = ctypes.cdll.LoadLibrary("./tisgrabber_x64.dll")
tis.declareFunctions(ic)

#ICImagingControlクラスライブラリを初期化します。
#この関数は、このライブラリの他の関数が呼び出される前に1回だけ呼び出す必要があります。
ic.IC_InitLibrary(0)


#########
# 解説2
#########
#ダイアログ画面を表示
hGrabber = ic.IC_ShowDeviceSelectionDialog(None)


#########
# 解説3
#########
#デバイスが有効か確認
if(ic.IC_IsDevValid(hGrabber)):
    ##### 解説3-1
    ##################
    # 露光時間
    ##################
    #自動露光時間をオフにする（露光時間を固定化するのに必須）
    exposureauto = ctypes.c_long()
    ic.IC_SetPropertySwitch(hGrabber, tis.T("Exposure"), tis.T("Auto"),
                            exposureauto)
    
    print("自動露光時間の設定： {0}".format(exposureauto.value))
    ic.IC_SetPropertySwitch(hGrabber, tis.T("Exposure"), tis.T("Auto"), 0)

    #露光時間を0.0303secに設定する
    ic.IC_SetPropertyAbsoluteValue(hGrabber, tis.T("Exposure"), tis.T("Value"),
                                   ctypes.c_float(0.0303))
    expmin = ctypes.c_float()
    expmax = ctypes.c_float()
    exposure = ctypes.c_float()
    #露光時間を0.0303secになっているか確認
    ic.IC_GetPropertyAbsoluteValue(hGrabber, tis.T("Exposure"), tis.T("Value"),
                                   exposure)
    ic.IC_GetPropertyAbsoluteValueRange(hGrabber, tis.T("Exposure"), tis.T("Value"),
                                        expmin, expmax)
    print("露光時間 ：{0}, 範囲： {1} ～ {2}".format(exposure.value,
                                                       expmin.value, expmax.value))
    
    ##### 解説3-2
    ##################
    # ゲイン
    ##################
    #自動ゲインをオフにする（ゲインを固定化するのに必須）
    gainauto = ctypes.c_long()
    ic.IC_SetPropertySwitch(hGrabber, tis.T("Gain"), tis.T("Auto"),
                            gainauto)
    
    print("自動ゲインの設定： {0}".format(gainauto.value))
    ic.IC_SetPropertySwitch(hGrabber, tis.T("Gain"), tis.T("Auto"), 0)

    #ゲインを10に設定する
    ic.IC_SetPropertyAbsoluteValue(hGrabber, tis.T("Gain"), tis.T("Value"),
                                   ctypes.c_float(10.0))
    gainmin = ctypes.c_long()
    gainmax = ctypes.c_long()
    gain = ctypes.c_float()
    ic.IC_GetPropertyAbsoluteValue(hGrabber, tis.T("Gain"), tis.T("Value"), gain)
    ic.IC_GetPropertyValueRange(hGrabber, tis.T("Gain"), tis.T("Value"),
                                gainmin, gainmax)
    print("ゲイン {0} 範囲： {1} ～ {2}".format(gain.value, gainmin.value, gainmax.value))

    ##### 解説3-3
    ##################
    # ホワイトバランス
    ##################
     #自動ホワイトバランスをオフにする（ホワイトバランスを固定化するのに必須）
    whiteBalanceauto = ctypes.c_long()
    ic.IC_SetPropertySwitch(hGrabber, tis.T("WhiteBalance"), tis.T("Auto"),
                            whiteBalanceauto)
    
    print("自動ホワイトバランスの設定： {0}".format(whiteBalanceauto.value))
    ic.IC_SetPropertySwitch(hGrabber, tis.T("WhiteBalance"), tis.T("Auto"), 0)

    #ホワイトバランスを10に設定する
    ic.IC_SetPropertyAbsoluteValue(hGrabber, tis.T("WhiteBalance"), tis.T("White Balance Red"),
                                   ctypes.c_float(2))
    ic.IC_SetPropertyAbsoluteValue(hGrabber, tis.T("WhiteBalance"), tis.T("White Balance Green"),
                                   ctypes.c_float(1))
    ic.IC_SetPropertyAbsoluteValue(hGrabber, tis.T("WhiteBalance"), tis.T("White Balance Blue"),
                                   ctypes.c_float(0.1))
    whiteBalanceRed = ctypes.c_float()
    whiteBalanceGreen = ctypes.c_float()
    whiteBalanceBlue = ctypes.c_float()
    ic.IC_GetPropertyAbsoluteValue(hGrabber, tis.T("WhiteBalance"), tis.T("White Balance Red"),
                                   whiteBalanceRed)
    ic.IC_GetPropertyAbsoluteValue(hGrabber, tis.T("WhiteBalance"), tis.T("White Balance Green"),
                                   whiteBalanceGreen)
    ic.IC_GetPropertyAbsoluteValue(hGrabber, tis.T("WhiteBalance"), tis.T("White Balance Blue"),
                                   whiteBalanceBlue)
    print("ホワイトバランス赤： {0} 緑： {1} 青： {2}".format(whiteBalanceRed.value, whiteBalanceGreen.value, whiteBalanceBlue.value))
    
    ##### 解説3-4
    # ソフトウェアトリガー
    ic.IC_PropertyOnePush(hGrabber, tis.T("Trigger"), tis.T("Software Trigger"))

    ##### 解説3-5
    #トリガーのGPIO読み取り
    inn = ctypes.c_long()
    inn_count=0
    while inn_count<3 :
        ic.IC_PropertyOnePush(hGrabber,tis.T("GPIO"),tis.T("Read") )
        ic.IC_GetPropertyValue(hGrabber,tis.T("GPIO"),tis.T("GP In"),inn )
        print("GPIO: %d" %(inn.value) )
        inn_count=inn_count+1
        time.sleep(1)

    ##### 解説3-6
    #フォーカス調整機能（ワンプッシュ）を実行する(オートフォーカスカメラのみ)
    ret = ic.IC_PropertyOnePush(hGrabber, tis.T("Focus"), tis.T("One Push"))
    if ret == -4:
        print("このカメラはフォーカス機能をサポートしていません。")

else:
    print("デバイスを開けませんでした。")

ic.IC_ReleaseGrabber(hGrabber)
